/***************************************************************************
 *   Copyright (C) 2006 by Philippe Lucidarme                              *
 *   http://www.ancr.org                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>


// Read a BMP file and displays the informations

// return -1 if the file does not exist
// return -2 if the signature is not a bmp file
// return -3 if the size of the file is not in accordance with the real size
// return -4 if a reserved byte is not equal to 0
// return -5 if the size of header structure is not equal to 40
// return -6 if the number of plane is not equal to 1
// return -7 if the number of bit by pixel is not standard (1,4,8 or 24)
// return -8 if the file is compressed

int ShowBMPFileProperties (char *filename)
{  
  unsigned int tmp=0; 
  
// File header

  unsigned short int signature;          // Signature of the file, should be equal to 0
  unsigned int size=0;                   // Size of the file (read in the header)
  unsigned int fsize=0;                  // Size of the file (real size of the file)
  unsigned int ImageOffset=0;                 // Offset of the image 

// Info header

  unsigned short int Nbits;				// Number of bits per pixel
  unsigned int width=0; 		// Width of the image
  unsigned int height=0; 		// Height of the image
  unsigned int SizeData=0;		// Size of the image 
  unsigned int HorResolution=0;		// Horizontal resolution
  unsigned int VerResolution=0;		// Vertical resolution
  unsigned int NumberOfColor=0;		// Number of colors in the image
  unsigned int NumberOfImColor=0;	// Number of important colors in the image

  FILE *file = fopen(filename,"r");
  if (file==NULL) return (-1);

  // Obtain file size.
  fseek (file , 0 , SEEK_END);
  fsize = ftell (file);
  rewind (file);

  // Read signature (Offset=0 Size=2) and check it is equal to "BM" (19778 when convert in INT
  fread (&signature,2,1,file);
  if (signature!=19778) return (-2);

  // Read size of file (Offset=2 Size=4) and check if it's equal to the real file size
  fread (&size,4,1,file);
  if (size!=fsize) return (-3);

  // Read reserved byte (Offset=6 Size=4) and check if it's equal to 0  
  fread (&tmp,4,1,file);
  if (tmp!=0) return (-4);

  // Read the offset (Offset=10 Size=4)
  fread (&ImageOffset,4,1,file);

  // Read the reserved bytes (Offset=14 Size=4) and check if it's equal to 40
  fread (&tmp,4,1,file);
  if (tmp!=40) return (-5);	

  // Read the image width (Offset=18 Size=4)
  fread (&width,4,1,file);

  // Read the image height (Offset=22 Size=4)
  fread (&height,sizeof(unsigned int),1,file);

  // Read the number of plane (Offset=26 Size=2) and check if it's equal to 1  
  fread (&tmp,2,1,file);
  if (tmp!=1) return (-6);

  // Read the number of bit by pixel (Offset=28 Size=2) and check if it's a valid value
  fread (&Nbits,2,1,file);  
  if ((Nbits!=1) && (Nbits!=4) && (Nbits!=8) && (Nbits!=24)) return (-7);

  // Read the compression type (Offset=30 Size=4) and check if it's a supported type  
  fread (&tmp,4,1,file);
  if (tmp!=0) return (-8);

  // Read the size of the image data (Offset=34 Size=4)
  fread (&SizeData,4,1,file);

  // Read the horizontal resolution of the image data (Offset=38 Size=4)  
  fread (&HorResolution,4,1,file);

  // Read the vertical resolution of the image data (Offset=42 Size=4)  
  fread (&VerResolution,4,1,file);

  // Read the number of colour in the image (Offset=46 Size=4)  
  fread (&NumberOfColor,4,1,file);

  // Read the number of important colour in the image (Offset=50 Size=4)
  fread (&NumberOfImColor,4,1,file);

 	// Displays the information contained in th header

	printf (" Signature : \t\t\tBM\n");
	printf (" Size of the file: \t\t%d bytes\n",size); 
	printf (" Offset of the data: \t\t%d\n",ImageOffset);   
	printf (" Width of the image: \t\t%d pixels\n",width);   
	printf (" Height of the image: \t\t%d pixels\n",height); 
	printf (" Number of bits by pixel : \t%d bits\n",Nbits); 
	printf (" Size of image data : \t\t%d octets\n",SizeData); 
	printf (" Horizontal resolution : \t%d ppm\n",HorResolution);  
	printf (" Vertical resolution : \t\t%d ppm\n",VerResolution);  
	printf (" Number of colors : \t\t%d color(s)\n",NumberOfColor);
	printf (" Number of important colors : \t%d color(s)\n",NumberOfImColor); 

	return (0);
}


int main(int argc, char *argv[])
{
	// Display Welcome message
	printf ("ANCR ::: BMP Properties (2006) ::: http://www.ancr.org\n\n");
	if (argc==1)
	{
 
		// No parameters, display help
		printf ("Enter a valid BMP file name in parameter\n");
		printf ("Example : BMPProp image.bmp\n\n");
		return (-1);
	}

	// If the function returned an error message, display the message :
  
	switch (ShowBMPFileProperties(argv[1]))
	{
		case  0 : printf ("\n"); break;
		case -1 : printf ("BMPProp : %s : No such file\n\n",argv[1]); break;
		case -2 : printf ("BMPProp : %s : Not a valid BMP file\n\n",argv[1]); break; 
		case -3 : printf ("BMPProp : %s : Size of the file in the header is not in accordance with the real size\n\n",argv[1]); break; 
		case -4 : printf ("BMPProp : %s : Wrong reserved bytes(Should be equal to zero)\n\n",argv[1]); break; 
		case -5 : printf ("BMPProp : %s : Wrong header size (Should be equal to 40)\n\n",argv[1]); break;
		case -6 : printf ("BMPProp : %s : Wrong number of planes (Should be equal to 1)\n\n",argv[1]); break;
		case -7 : printf ("BMPProp : %s : Wrong number of bits per pixel (Should be equal to 1,4,8 or 24)\n\n",argv[1]); break;
		case -8 : printf ("BMPProp : %s : The file is compressed (Not supported by this program)\n\n",argv[1]); break;
    	}

  return (0);
}
